%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%EE 658 Project
%Watermarking using Color Quantization technique
%by Dien Nguyen
%The code asks to choose 1) an host image
%                        2) a watermark image
%                        3) size of the watermark image
%Note: images such as flowers.tif, football.tif, and circles.tif
%are only in matlab 6.5
%output also includes the PSNR for RGB
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Color_watermarking
clear
clear all

Host_Image=input('Select a host image:1=flowers, 2=prof. fred harris, 3=Dr. Abut, 4=Escher, 5=Lena, 6=football, 7=Homer J, 8=Sesame: ');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

%HOST IMAGES
if Host_Image==1
IM_h = imread('flowers.tif');
end
if Host_Image==2
IM_h = imread('fharris.jpg');
end
if Host_Image==3
IM_h = imread('abut4.jpg');
end
if Host_Image==4
IM_h = imread('LW378.jpg');
end
if Host_Image==5
IM_h = imread('lena.tif');
end
%IM_h = imread('Escher1.jpg');
if Host_Image==6
IM_h = imread('football.jpg');
end
if Host_Image==7
    IM_h=imread('Homer.jpg');
end
if Host_Image==8
    IM_h=imread('Sesame.jpg');
end

N=512;
IM_h = imresize(IM_h, [N N]); %resize the image to N X N pixels
IM_h = im2double(IM_h); %converting to double
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%water mark images
%s1=32;
%s2=32; %sizes of the watermark image

Watermark=input('Select a watermark image:1=Cicles, 2=Lizards, 3=IEEE, 4=?: ');
if Watermark==1
IM_w = imread('circles.tif'); %watermark image: binary image
end
if Watermark==2
IM_w=imread('Escher2.tif');
end
if Watermark==3
IM_w = imread('P.tif'); %ieee sympol
end
if Watermark==4
    IM_w=imread('ee658.tif');
end

%choosing the size
Size = input('Select a dimension for the watermark (16, 32, 64): ');
s1=Size;
s2=Size;
IM_w = imresize(IM_w, [s1 s2]);
IM_w = im2double(IM_w);
%In practice, the watermark image will be scrambled
%either by DES or RSA encryption algorithm
%for extra protection against attacks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Color palette
%Color palette usually generated by Photoshop
%using maximum colors and using PCA to reorder the colors
%so that the closest colors are next to one another
%For this project, the colorcube generated by Matlab was used.
%It gives extremely good results
C_pal = colorcube(256); %colorcube is of matlab
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Main program according to the paper by CHANG et al
[m,n,k]=size(IM_h);

%using PRNG to determine 64 x 64 of the host image that we 
%want to imbed the watermark

%for example, the first 64 pixels of the host image
S=10; %seed of the random number generator
rand('seed',S); %seeding the random generator
row=abs(floor(rand(1,s1)*(N-1)))+1; %randomizing the indexes to be watermark
col=abs(floor(rand(1,s2)*(N-1)))+1;

X=zeros(256,3);
QQ = zeros(256,1);
IM_q=IM_h;
ID=zeros(s1,s2);
k=1;

for mm=1:s1
    for nn=1:s2
        Y=[IM_h(row(mm),col(nn),1),IM_h(row(mm),col(nn),2),IM_h(row(mm),col(nn),3)];
        X(1:256,1)=Y(1);
        X(1:256,2)=Y(2);
        X(1:256,3)=Y(3);
        
        %finding the closest color from the color palette
        D=(sum((X-C_pal).^2,2)); %Euclidean distance: SED
        [M,In_dex]=min(D); %getting the index from the color palette        
        MIN(k)=M;
        C_table(mm)=In_dex; %storing the indexes
        %for pp=1:s1
        %for qq=1:s2
        T=mod(IM_w(mm,nn) + In_dex,2);
        if T==0
            ID(mm,nn)=In_dex;
        else
            QQ(In_dex)=100; %making it big
            D2 = D + QQ; %adding QQ to get the new sum, min at Id_dex is removed
            [M2, Index_next]=min(D2); %index of the next minimum
            T2 = mod(IM_w(mm,nn)+Index_next,2);
            while T2~=0
                QQ(Index_next)=100;
                D2=D +QQ;
                [M2,Index_next]=min(D2);
                T2=mod(IM_w(mm,nn)+Index_next,2);
            end
            T3 = abs(M2 - M);
            if T3 <= 0.05 %threshold
                ID(mm,nn)=Index_next;
            else
                h1 = [IM_h(col(mm)-1,row(nn),1),IM_h(col(mm)-1,row(nn),2),IM_h(col(mm)-1,row(nn),2)]; %immediate left pixel
                ha = [IM_h(col(mm),row(nn)-1,1),IM_h(col(mm),row(nn)-1,2),IM_h(col(mm),row(nn)-1,3)]; %pixel above;
                h_mean = (h1 +ha)/2; %the average
                D3=(sum((Y-h_mean).^2,2));
                if D3 < M2
                    if IM_w(mm,nn)==0
                        ID(mm,nn)=1; %storing index 0
                    else
                        ID(mm,nn)=256; %storing index 255
                    end
                else
                    ID(mm,nn)=Index_next;
                end
            end
        end
        
        
        %forming the quantized image
        IM_q(row(mm),col(nn),1:3)=C_pal(ID(mm,nn),1:3); %Index between 0 and 255
        
        QQ=zeros(256,1); %resetting
        
        k=k+1;
    end  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plotting the results
figure(1)
imshow(IM_h)
title('Host image')
figure(2)
imshow(IM_q)
title('Watermarked image')

%Extracting process

%Water_m=mod(ID,2);

%This algorithm also give the same Water_m
%Water_m=zeros(s1,s2);
rand('seed',S); %seeding the random generator
row2=abs(floor(rand(1,s1)*(N-1)))+1; %randomizing the indexes to be watermark
col2=abs(floor(rand(1,s2)*(N-1)))+1;


for kk=1:s1
    for uu=1:s2
       TT=IM_q(row2(kk),col2(uu),1:3); %locating the embedded pix
        XR(1:256,1)=TT(1);
        XR(1:256,2)=TT(2);
        XR(1:256,3)=TT(3);
        DR=(sum((XR-C_pal).^2,2));
        [MR,IDR]=min(DR); %recovering the index
        I_tab(kk,uu)=IDR;
    end
end

Water_m=mod(I_tab,2);
Water_m=im2uint8(Water_m);

figure(3)
imshow(IM_w)
title('Watermark image')

figure(4)
imshow(Water_m)
title('Extracted watermark')

figure(5)
subplot(2,1,1)
imshow(IM_h)
title('Host image')
subplot(2,1,2)
imshow(IM_q)
title('Watermarked image')

figure(6)
subplot(2,1,1)
imshow(IM_w)
title('Watermark image')
subplot(2,1,2)
imshow(Water_m)
title('Extracted watermark')

figure(7)
subplot(2,1,1)
stem(C_table)
title('Indexes from the color palette')
grid
subplot(2,1,2)
Err=im2double(IM_h)-im2double(IM_q);
for tt=1:N
    for pp=1:N
        BB=[Err(tt,pp,1) Err(tt,pp,2), Err(tt,pp,1)];
        if sum(BB)~=0
            Err(tt,pp,1:3)=[0 0 0];
        else
            Err(tt,pp,1:3)=~Err(tt,pp,1:3);
        end
    end
end
            
        
Err=im2uint8(Err);
imshow(Err)
title('Difference bewteen Watermarked and Host image')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%
%calculating the PSNR
%PSNR(IM_q,IM_h)

[m,n]=size(IM_q);

%calculate MSE
err=IM_h-IM_q;
mse=sum(sum(err.^2))/(m*n);

%calculate RMSE
RMSE=sqrt(mse);

%calculate PSNR
PSNR=20*log10(1/RMSE);
sprintf('R = %5.2f(dB)',PSNR(:,:,1)) 
sprintf('G = %5.2f(dB)',PSNR(:,:,2))
sprintf('B = %5.2f(dB)',PSNR(:,:,3))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



