% Digital watermarking
%Ilias Faqir & Dien Nguyen



function watermark
clear all
close all

clc
[OrgName,p1name] = uigetfile('*.bmp','Original Picture');
[WatName,p2name] = uigetfile('*.bmp','Watermark Picture');
Bits=menu('Please Select # of Bits','0','1','2','3','4','5','6','7','8');
Choice1=[0,1,2,3,4,5,6,7,8];
Compression=menu('Please Select the Compression','100','75','50','25','1');
Choice2=[0,25,50,75,100];
Original = imread(OrgName, 'bmp');		
Watermark = imread(WatName, 'bmp');																							%# of bits to replace (0-8)
Shift=bitshift(Watermark,(Choice1(Bits)-8));
Clear=double(Original);
for i = 1:Choice1(Bits)
    fprintf(1,'.');
    Clear=bitset(Clear, i, 0);
end
A=double(Shift);
Watermarked=uint8(round(A+Clear));
showpic('Watermarked Image',Watermarked)
imwrite(Watermarked,'web2.jpg','jpg');	
showpic('Original Image',Original)
imwrite(Original,'web1.jpg','jpg');					
Watermark1=bitshift(Watermarked,8-Choice1(Bits));
showpic('Extracted Watermark',Watermark1)
imwrite(Watermark1,'web3.jpg','jpg');		
imwrite(Watermarked,'Watermarked.jpg','jpg','Quality',Choice2(Compression));		
JPEG=imread('Watermarked.jpg','jpg');			
Watermark2=bitshift(JPEG,8-Choice1(Bits));
showpic('Extracted Watermark After JPEG Compression',Watermark2)
imwrite(Watermark2,'web4.jpg','jpg');
A=num2str(corr2(Original,Watermarked));
['The Correlation between the Original Image and the Watermarked Image is: ',A]
B=num2str(corr2(Watermark1,Watermark));
['The Correlation between the Original Watermark and the Extracted Watermark is: ',B]
C=num2str(corr2(Watermark2,Watermark));
['The Correlation between the Original Watermark & Extracted Watermark after Compression is: ',C]

%----------------------------------------------
function r = corr2(a,b)

error(nargchk(2,2,nargin));

if any(size(a)~=size(b)), error('A and B must be the same size.'); end

if (~isa(a,'double'))
    a = double(a);
end

if (~isa(b,'double'))
    b = double(b);
end

a = a - mean2(a);
b = b - mean2(b);

r = sum(sum(a.*b))/sqrt(sum(sum(a.*a))*sum(sum(b.*b)));


%---------------------------------------------------
function y = mean2(x)

y = sum(x(:))/prod(size(x));


%----------------------------------------------------
function s = num2str(x, f)

if isstr(x)
   s = x;
   return
end

if nargin < 2 & ~isempty(x) & all(all(x==fix(x)))
   d = min(12,max(1,max(ceil(log10(abs(x(:))+(x(:)==0))))));
   f = ['%' sprintf('%d',d+2) 'd'];
   fi = ['%-' sprintf('%d',d+2) 's'];
elseif nargin < 2
   d = min(11,max(1,max(ceil(log10(abs(x(:))+(x(:)==0))))))+4;
   f = ['%' int2str(d+7) '.' int2str(d) 'g'];
   fi = ['%-' int2str(d+7) 's'];
elseif ~isstr(f)
   fi = ['%-' int2str(f+7) 's'];
   f = ['%' int2str(f+7) '.' int2str(f) 'g'];
else
  % Sanity check on format
  k = find(f=='%');
  if isempty(k), error(sprintf('''%s'' is an invalid format.',f)); end
  d = sscanf(f(k(1)+1:end),'%f');
  if isempty(d), error('Format must contain field width.'); end
  fi = ['%-' int2str(d) 's'];
end
   
[m,n] = size(x);
s = '';
for i = 1:m,
   t = [];
   for j = 1:n,
      u = sprintf(f, real(x(i,j)));
      if ~isreal(x) & imag(x(i,j)) == 0,
          u = [u '+' formatimag(f,fi,0)];
      elseif imag(x(i,j)) > 0
          u = [u '+' formatimag(f,fi,imag(x(i,j)))];
      elseif imag(x(i,j)) < 0
          u = [u '-' formatimag(f,fi,-imag(x(i,j)))];
      end
      t = [t u];
   end
   s = strvcat(s,t);
end

s = lefttrim(s);

% If it's a scalar remove the trailing blanks too.
if length(x)==1,
  s = deblank(s);
end

%-----------------------
function v = formatimag(f,fi,x)
% Format imaginary part
v = [sprintf(f,x) 'i'];
v = lefttrim(v);
v = sprintf(fi,v);


%-----------------------
function s = lefttrim(s)
% Remove leading blanks
if ~isempty(s)
  [r,c] = find(s ~= ' ');
  if ~isempty(c)
    s = s(:,min(c):end);
  end
end



%---------------------------------------------------------
%Displays Picture
function showpic(title,image)

figure('Name',title);
imagesc(image);
colormap(gray(256));

